<?php

namespace Amazingcat\Labels\Block\Product\View;

use Amazingcat\Labels\Model\ResourceModel\DefaultLabel\CollectionFactory as LabelCollectionFactory;
use Magento\Catalog\Block\Product\Context;
use Magento\Catalog\Block\Product\View\Gallery;
use Magento\Catalog\Model\Product\Gallery\ImagesConfigFactoryInterface;
use Magento\Catalog\Model\Product\Image\UrlBuilder;
use Magento\Catalog\Model\ProductRepository;
use Magento\CatalogRule\Model\ResourceModel\Rule;
use Magento\CatalogRule\Model\ResourceModel\Rule\CollectionFactory;
use Magento\Customer\Model\Session;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\Json\EncoderInterface;
use Magento\Framework\Stdlib\ArrayUtils;
use Magento\Store\Model\StoreManagerInterface;
use Psr\Log\LoggerInterface;
use Throwable;

/**
 * Class Label
 * @package Amazingcat\Labels\Block\Product\View
 */
class Label extends Gallery
{
    /**
     * @var Rule
     */
    protected $rule;

    /**
     * @var StoreManagerInterface
     */
    protected $storeManager;

    /**
     * @var Session
     */
    protected $session;

    /**
     * @var CollectionFactory
     */
    protected $ruleCollectionFactory;

    /**
     * @var ProductRepository
     */
    protected $productRepository;

    /**
     * @var LabelCollectionFactory
     */
    protected $defaultLabelCollectionFactory;

    /**
     * @var LoggerInterface
     */
    protected $logger;

    /**
     * Label constructor.
     * @param Context $context
     * @param ArrayUtils $arrayUtils
     * @param EncoderInterface $jsonEncoder
     * @param array $data
     * @param ImagesConfigFactoryInterface|null $imagesConfigFactory
     * @param array $galleryImagesConfig
     * @param UrlBuilder|null $urlBuilder
     * @param Rule $rule
     * @param StoreManagerInterface $storeManager
     * @param Session $session
     * @param CollectionFactory $ruleCollectionFactory
     * @param ProductRepository $productRepository
     * @param LabelCollectionFactory $defaultLabelCollectionFactory
     * @param LoggerInterface $logger
     */
    public function __construct(
        Context $context,
        ArrayUtils $arrayUtils,
        EncoderInterface $jsonEncoder,
        array $data = [],
        ?ImagesConfigFactoryInterface $imagesConfigFactory = null,
        array $galleryImagesConfig = [],
        ?UrlBuilder $urlBuilder = null,
        Rule $rule,
        StoreManagerInterface $storeManager,
        Session $session,
        CollectionFactory $ruleCollectionFactory,
        ProductRepository $productRepository,
        LabelCollectionFactory $defaultLabelCollectionFactory,
        LoggerInterface $logger
    ) {
        $this->rule = $rule;
        $this->storeManager = $storeManager;
        $this->session = $session;
        $this->ruleCollectionFactory = $ruleCollectionFactory;
        $this->productRepository = $productRepository;
        $this->defaultLabelCollectionFactory = $defaultLabelCollectionFactory;
        $this->logger = $logger;

        parent::__construct(
            $context,
            $arrayUtils,
            $jsonEncoder,
            $data,
            $imagesConfigFactory,
            $galleryImagesConfig,
            $urlBuilder
        );
    }

    /**
     * Get product label.
     * @return array|null
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function getLabel()
    {
        $product = $this->getProduct();
        $productRules = $this->rule->getRulesFromProduct(
            time(),
            $this->storeManager->getStore()->getWebsiteId(),
            $this->session->getCustomerGroupId(),
            $product->getId()
        );

        if (empty($productRules)) {
            return null;
        }

        $response = [
            'label' => '',
            'position' => 0,
        ];

        try {
            foreach ($productRules as $productRule) {
                $ruleRecord = $this->ruleCollectionFactory->create()
                    ->addFieldToFilter('rule_id', $productRule['rule_id'])
                    ->addFieldToFilter('is_active', 1)
                    ->getFirstItem();

                if ($ruleRecord->getRuleLabel()) {
                    $ruleBadge = $ruleRecord->getRuleLabel();
                }
            }

            if (!empty($ruleBadge)) {
                $response['label'] = $ruleBadge;

                $this->_eventManager->dispatch('amazingcat_labels_render_label', ['label' => $response]);

                return $response;
            } elseif (!empty($product->getProductLabel()) && $product->getProductLabel() != 'no_selection') {
                $productLabel = $product->getProductLabel();
                $response['label'] = '/pub/media/catalog/product' . $productLabel;

                $this->_eventManager->dispatch('amazingcat_labels_render_label', ['label' => $response]);

                return $response;
            } else {
                $productReloaded = $this->productRepository->getById($product->getId(), false, null, true);
                $defaultLabel = $productReloaded->getDefaultLabel();

                if ($defaultLabel == 0) {
                    return null;
                }

                $collection = $this->defaultLabelCollectionFactory->create();
                $defaultLabel = $collection
                    ->addFieldToFilter('id', $defaultLabel)
                    ->getFirstItem();
                if (!empty($url = $defaultLabel->getImagePath())) {
                    $imagePath = '/pub' . $defaultLabel->getImagePath();
                    $response = [
                        'label' => $imagePath,
                        'position' => $defaultLabel->getPosition(),
                    ];
                } else {
                    return null;
                }

                $this->_eventManager->dispatch('amazingcat_labels_render_label', ['label' => $response]);

                return $response;
            }
        } catch (NoSuchEntityException $e) {
            $this->logger->critical($e->getFile() . ": " . $e->getMessage());

            return null;
        } catch (Throwable $e) {
            $this->logger->critical($e->getFile() . ": " . $e->getMessage());

            return null;
        }
    }
}
